import {
  CSSProperties,
  useEffect,
  useMemo,
  useState,
} from "react";
import { randomString } from "./utils";
import { IAnimationProps, IAnimationState } from "./types";

export interface IAnimationDefinition {
  definition: string;
  name: string;
}

export const generateAnimationName = (): string => {
  return randomString(32);
};

export const styleToString = (css: CSSProperties): string => {
  return Object.entries(css)
    .map(([k, v]) => `${k}:${v}`)
    .join(";");
};

export const generateAnimation = (
  state: IAnimationState,
  duration: number = 1
): IAnimationDefinition => {
  const keys = Object.keys(state);
  const values = keys
    .map((key) => {
      return `${key} { ${styleToString(state[key])}; }`;
    })
    .join("\n");

  const animationName = generateAnimationName();

  const template = `
        @keyframes ${animationName} {
          ${values}
    }
    .${animationName} {
       animation: ${animationName} ${duration}s forwards;
    }
  `;

  return { definition: template, name: animationName };
};

export const Animation = (props: IAnimationProps) => {
  const { state, children, duration = 1 } = props;
  const [animationDefinition, setAnimationDefinition] =
    useState<IAnimationDefinition>({ definition: "", name: "" });
  const [animationName, setAnimationName] = useState<string | null>();

  const firstState = useMemo(() => {
    if (!state) return undefined;
    const keys = Object.keys(state);
    if (keys.length === 0) return undefined;
    return state[keys[0]];
  }, [state]);

  useEffect(() => {
    if (!state) return;
    if (animationName) return;

    const animationDef = generateAnimation(state, duration);
    if (animationDef.definition !== animationDefinition.definition) {
      setAnimationDefinition(animationDef);
      setAnimationName(animationDef.name);
    }
  }, [
    state,
    animationDefinition,
    setAnimationDefinition,
    setAnimationName,
    duration,
  ]);

  return (
    <>
      <style>{animationDefinition.definition}</style>
      <div style={firstState} className={animationName || ""}>
        {children}
      </div>
    </>
  );
};
